@@ -1,4 +1,5 @@
 /*
+ *  Copyright (C) 2015, 2018 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009-2013 Sourcefire, Inc.
  *  Author: Tomasz Kojm <tkojm@clamav.net>
  *
@@ -199,11 +200,11 @@ static int printconf(const char *name)
 static void help(void)
 {
     printf("\n");
-    printf("           Clam AntiVirus: Configuration Tool %s\n", get_version());
-    printf("           By The ClamAV Team: http://www.clamav.net/about.html#credits\n");
-    printf("           (C) 2009 Sourcefire, Inc.\n\n");
-
-    printf("    --help                 -h         Show help\n");
+    printf("                       Clam AntiVirus: Configuration Tool %s\n", get_version());
+    printf("           By The ClamAV Team: https://www.clamav.net/about.html#credits\n");
+    printf("           (C) 2009-2018 Cisco Systems, Inc.\n");
+    printf("\n");
+    printf("    --help                 -h         Show this help\n");
     printf("    --version              -V         Show version\n");
     printf("    --config-dir=DIR       -c DIR     Read configuration files from DIR\n");
     printf("    --non-default          -n         Only display non-default settings\n");
@@ -457,9 +458,17 @@ int main(int argc, char **argv)
 #ifdef HAVE_BZLIB_H
 	printf("BZIP2 ");
 #endif
+
 #ifdef HAVE_LIBXML2
 	printf("LIBXML2 ");
 #endif
+#ifdef HAVE_PCRE
+#if USING_PCRE2
+	printf("PCRE2 ");
+#else
+	printf("PCRE ");
+#endif
+#endif
 #ifdef HAVE_ICONV
 	printf("ICONV ");
 #endif
