@@ -671,6 +671,7 @@ static inline int parsehwp3_docsummary(cli_ctx *ctx, off_t offset)
 static inline int parsehwp3_paragraph(cli_ctx *ctx, fmap_t *map, int p, int level, off_t *roffset, int *last)
 {
     off_t offset = *roffset;
+    off_t new_offset;
     uint16_t nchars, nlines, content;
     uint8_t ppfs, ifsc, cfsb;
     int i, c, l, sp = 0, term = 0, ret = CL_SUCCESS;
@@ -780,7 +781,12 @@ static inline int parsehwp3_paragraph(cli_ctx *ctx, fmap_t *map, int p, int leve
         offset += HWP3_LINEINFO_SIZE;
     }
 #else
-    offset += (nlines * HWP3_LINEINFO_SIZE);
+    new_offset = offset + (nlines * HWP3_LINEINFO_SIZE);
+    if ((new_offset < offset) || (new_offset >= map->len)) {
+        cli_errmsg("HWP3.x: Paragraph[%d, %d]: length value is too high, invalid. %u\n", level, p, nlines);
+        return CL_EPARSE;
+    }
+    offset = new_offset;
 #endif
 
     if (offset >= map->len)
@@ -833,7 +839,10 @@ static inline int parsehwp3_paragraph(cli_ctx *ctx, fmap_t *map, int p, int leve
         hwp3_debug("HWP3.x: Paragraph[%d, %d]: content starts @ offset %llu\n", level, p, (long long unsigned)offset);
 
     /* scan for end-of-paragraph [0x0d00 on offset parity to current content] */
-    while (!term) {
+    while ((!term) &&
+           (offset >= 0) &&
+           (offset < map->len))
+    {
         if (fmap_readn(map, &content, offset, sizeof(content)) != sizeof(content))
             return CL_EREAD;
 
@@ -872,7 +881,13 @@ static inline int parsehwp3_paragraph(cli_ctx *ctx, fmap_t *map, int p, int leve
                         return CL_EREAD;
 
                     length = le32_to_host(length);
-                    offset += (8 + length);
+                    new_offset = offset + (8 + length);
+                    if ((new_offset <= offset) || (new_offset >= map->len)) {
+                        cli_errmsg("HWP3.x: Paragraph[%d, %d]: length value is too high, invalid. %u\n", level, p, length);
+                        return CL_EPARSE;
+                    }
+                    offset = new_offset;
+
 #if HWP3_DEBUG
                     cli_errmsg("HWP3.x: Paragraph[%d, %d]: possible invalid usage of reserved special character %u\n", level, p, content);
                     return CL_EFORMAT;
@@ -899,7 +914,12 @@ static inline int parsehwp3_paragraph(cli_ctx *ctx, fmap_t *map, int p, int leve
                         return CL_EREAD;
 
                     length = le32_to_host(length);
-                    offset += (8 + length);
+                    new_offset = offset + (8 + length);
+                    if ((new_offset <= offset) || (new_offset >= map->len)) {
+                        cli_errmsg("HWP3.x: Paragraph[%d, %d]: length value is too high, invalid. %u\n", level, p, length);
+                        return CL_EPARSE;
+                    }
+                    offset = new_offset;
                     break;
                 }
             case 6: /* bookmark */
@@ -1038,7 +1058,13 @@ static inline int parsehwp3_paragraph(cli_ctx *ctx, fmap_t *map, int p, int leve
 
                     /* cell information (27 bytes x ncells(offset 80 of table)) */
                     hwp3_debug("HWP3.x: Paragraph[%d, %d]: box cell info array starts @ %llu\n", level, p, (long long unsigned)offset);
-                    offset += (27 * ncells);
+
+                    new_offset = offset + (27 * ncells);
+                    if ((new_offset < offset) || (new_offset >= map->len)) {
+                        cli_errmsg("HWP3.x: Paragraph[%d, %d]: number of box cells is too high, invalid. %u\n", level, p, ncells);
+                        return CL_EPARSE;
+                    }
+                    offset = new_offset;
 
                     /* cell paragraph list */
                     hwp3_debug("HWP3.x: Paragraph[%d, %d]: box cell paragraph list starts @ %llu\n", level, p, (long long unsigned)offset);
@@ -1079,7 +1105,12 @@ static inline int parsehwp3_paragraph(cli_ctx *ctx, fmap_t *map, int p, int leve
                     hwp3_debug("HWP3.x: Paragraph[%d, %d]: drawing is %u additional bytes\n", level, p, size);
 
                     size = le32_to_host(size);
-                    offset += (348 + size);
+                    new_offset = offset + (348 + size);
+                    if ((new_offset <= offset) || (new_offset >= map->len)) {
+                        cli_errmsg("HWP3.x: Paragraph[%d, %d]: image size value is too high, invalid. %u\n", level, p, size);
+                        return CL_EPARSE;
+                    }
+                    offset = new_offset;
 
                     /* caption paragraph list */
                     hwp3_debug("HWP3.x: Paragraph[%d, %d]: drawing caption paragraph list starts @ %llu\n", level, p, (long long unsigned)offset);
@@ -1428,8 +1459,12 @@ static inline int parsehwp3_paragraph(cli_ctx *ctx, fmap_t *map, int p, int leve
                         return CL_EREAD;
 
                     length = le32_to_host(length);
-
-                    offset += (8 + length);
+                    new_offset = offset + (8 + length);
+                    if ((new_offset <= offset) || (new_offset >= map->len)) {
+                        cli_errmsg("HWP3.x: Paragraph[%d, %d]: length value is too high, invalid. %u\n", level, p, length);
+                        return CL_EPARSE;
+                    }
+                    offset = new_offset;
                     break;
                 }
             case 30: /* bundle of blanks (ON SALE for 2.99!) */
@@ -1725,7 +1760,7 @@ static inline int parsehwp3_infoblk_1(cli_ctx *ctx, fmap_t *dmap, off_t *offset,
 static int hwp3_cb(void *cbdata, int fd, cli_ctx *ctx)
 {
     fmap_t *map, *dmap;
-    off_t offset, start;
+    off_t offset, start, new_offset;
     int i, t = 0, p = 0, last = 0, ret = CL_SUCCESS;
     uint16_t nstyles;
 #if HAVE_JSON
@@ -1779,8 +1814,12 @@ static int hwp3_cb(void *cbdata, int fd, cli_ctx *ctx)
             cli_jsonint(fonts, NULL, nfonts);
 #endif
         hwp3_debug("HWP3.x: Font Entry %d with %u entries @ offset %llu\n", i+1, nfonts, (long long unsigned)offset);
-
-        offset += (2 + nfonts * 40);
+        new_offset = offset + (2 + nfonts * 40);
+        if ((new_offset <= offset) || (new_offset >= map->len)) {
+            cli_errmsg("HWP3.x: Font Entry: number of fonts is too high, invalid. %u\n", nfonts);
+            return CL_EPARSE;
+        }
+        offset = new_offset;
     }
 
     /* Styles - 2 + (n x 238) bytes where n is the first 2 bytes of the section */
@@ -1796,7 +1835,11 @@ static int hwp3_cb(void *cbdata, int fd, cli_ctx *ctx)
         cli_jsonint(ctx->wrkproperty, "StyleCount", nstyles);
 #endif
     hwp3_debug("HWP3.x: %u Styles @ offset %llu\n", nstyles, (long long unsigned)offset);
-
+    new_offset = offset + (2 + nstyles * 238);
+    if ((new_offset <= offset) || (new_offset >= map->len)) {
+        cli_errmsg("HWP3.x: Font Entry: number of font styles is too high, invalid. %u\n", nstyles);
+        return CL_EPARSE;
+    }
     offset += (2 + nstyles * 238);
 
     last = 0;
@@ -1839,7 +1882,8 @@ int cli_scanhwp3(cli_ctx *ctx)
 {
     struct hwp3_docinfo docinfo;
     int ret = CL_SUCCESS;
-    off_t offset = 0;
+    off_t offset = 0, new_offset = 0;
+    fmap_t *map = *ctx->fmap;
 
 #if HAVE_JSON
     /*
@@ -1870,7 +1914,12 @@ int cli_scanhwp3(cli_ctx *ctx)
 
     if (docinfo.di_infoblksize) {
         /* OPTIONAL TODO: HANDLE OPTIONAL INFORMATION BLOCK #0's FOR PRECLASS */
-        offset += docinfo.di_infoblksize;
+        new_offset = offset + docinfo.di_infoblksize;
+        if ((new_offset <= offset) || (new_offset >= map->len)) {
+            cli_errmsg("HWP3.x: Doc info block size is too high, invalid. %u\n", docinfo.di_infoblksize);
+            return CL_EPARSE;
+        }
+        offset = new_offset;
     }
 
     if (docinfo.di_compressed)
