@@ -1,5 +1,7 @@
 /*
+ *  Copyright (C) 2015, 2017 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
+ *  All Rights Reserved.
  *
  *  Authors: Tomasz Kojm
  *
@@ -34,6 +36,7 @@
 #include "others.h"
 #include "matcher-ac.h"
 #include "matcher-bm.h"
+#include "matcher-pcre.h"
 #include "filetypes.h"
 #include "matcher.h"
 #include "pe.h"
@@ -52,6 +55,10 @@
 #include "perflogging.h"
 #include "bytecode_priv.h"
 #include "bytecode_api_impl.h"
+#ifdef HAVE_YARA
+#include "yara_clam.h"
+#include "yara_exec.h"
+#endif
 
 #ifdef CLI_PERF_LOGGING
 
@@ -95,20 +102,20 @@ static inline int matcher_run(const struct cli_matcher *root,
 			      cli_file_t ftype,
 			      struct cli_matched_type **ftoffset,
 			      unsigned int acmode,
+                              unsigned int pcremode,
 			      struct cli_ac_result **acres,
 			      fmap_t *map,
 			      struct cli_bm_off *offdata,
+			      struct cli_pcre_off *poffdata,
 			      cli_ctx *ctx)
 {
-    int ret;
+    int ret, saved_ret = CL_CLEAN;
     int32_t pos = 0;
     struct filter_match_info info;
     uint32_t orig_length, orig_offset;
     const unsigned char* orig_buffer;
     unsigned int viruses_found = 0;
 
-    UNUSEDPARAM(map);
-
     if (root->filter) {
 	if(filter_search_ext(root->filter, buffer, length, &info) == -1) {
 	    /*  for safety always scan last maxpatlen bytes */
@@ -150,19 +157,84 @@ static inline int matcher_run(const struct cli_matcher *root,
 	    if (SCAN_ALL)
 		viruses_found = 1;
 	    else {
-		cli_append_virus(ctx, *virname);
-		return ret;
+		ret = cli_append_virus(ctx, *virname);
+		if (ret != CL_CLEAN)
+                    return ret;
 	    }
 	}
     }
     PERF_LOG_TRIES(acmode, 0, length);
     ret = cli_ac_scanbuff(buffer, length, virname, NULL, acres, root, mdata, offset, ftype, ftoffset, acmode, ctx);
+    if (ret != CL_CLEAN) {
+        if (ret == CL_VIRUS) {
+            if (SCAN_ALL)
+                viruses_found = 1;
+            else {
+                ret = cli_append_virus(ctx, *virname);
+                if (ret != CL_CLEAN)
+                    return ret;
+            }
+        } else if (ret > CL_TYPENO && acmode & AC_SCAN_VIR)
+            saved_ret = ret;
+        else
+            return ret;
+    }
+
+    /* due to logical triggered, pcres cannot be evaluated until after full subsig matching */
+    /* cannot save pcre execution state without possible evasion; must scan entire buffer */
+    /* however, scanning the whole buffer may require the whole buffer being loaded into memory */
+#if HAVE_PCRE
+    if (root->pcre_metas) {
+        int rc;
+        uint64_t maxfilesize;
+
+        if (map && (pcremode == PCRE_SCAN_FMAP)) {
+            if (offset+length >= map->len) {
+                /* check that scanned map does not exceed pcre maxfilesize limit */
+                maxfilesize = (uint64_t)cl_engine_get_num(ctx->engine, CL_ENGINE_PCRE_MAX_FILESIZE, &rc);
+                if (rc != CL_SUCCESS)
+                    return rc;
+                if (maxfilesize && (map->len > maxfilesize)) {
+                    cli_dbgmsg("matcher_run: pcre max filesize (map) exceeded (limit: %llu, needed: %llu)\n",
+                               (long long unsigned)maxfilesize, (long long unsigned)map->len);
+                    return CL_EMAXSIZE;
+                }
+
+                cli_dbgmsg("matcher_run: performing regex matching on full map: %u+%u(%u) >= %zu\n", offset, length, offset+length, map->len);
+
+                buffer = fmap_need_off_once(map, 0, map->len);
+                if (!buffer)
+                    return CL_EMEM;
+
+                /* scan the full buffer */
+                ret = cli_pcre_scanbuf(buffer, map->len, virname, acres, root, mdata, poffdata, ctx);
+            }
+        }
+        else if (pcremode == PCRE_SCAN_BUFF) {
+            /* check that scanned buffer does not exceed pcre maxfilesize limit */
+            maxfilesize = (uint64_t)cl_engine_get_num(ctx->engine, CL_ENGINE_PCRE_MAX_FILESIZE, &rc);
+            if (rc != CL_SUCCESS)
+                return rc;
+            if (maxfilesize && (length > maxfilesize)) {
+                cli_dbgmsg("matcher_run: pcre max filesize (buf) exceeded (limit: %llu, needed: %u)\n", (long long unsigned)maxfilesize, length);
+                return CL_EMAXSIZE;
+            }
+
+            cli_dbgmsg("matcher_run: performing regex matching on buffer with no map: %u+%u(%u)\n", offset, length, offset+length);
+            /* scan the specified buffer */
+            ret = cli_pcre_scanbuf(buffer, length, virname, acres, root, mdata, poffdata, ctx);
+        }
+    }
+#endif /* HAVE_PCRE */
+    /* end experimental fragment */
 
     if (ctx && !SCAN_ALL && ret == CL_VIRUS)
-	cli_append_virus(ctx, *virname);
+        return cli_append_virus(ctx, *virname);
     if (ctx && SCAN_ALL && viruses_found)
-	return CL_VIRUS;
+        return CL_VIRUS;
 
+    if (saved_ret && ret == CL_CLEAN)
+        return saved_ret;
     return ret;
 }
 
@@ -199,7 +271,7 @@ int cli_scanbuff(const unsigned char *buffer, uint32_t length, uint32_t offset,
 	if(!acdata && (ret = cli_ac_initdata(&mdata, troot->ac_partsigs, troot->ac_lsigs, troot->ac_reloff_num, CLI_DEFAULT_AC_TRACKLEN)))
 	    return ret;
 
-	ret = matcher_run(troot, buffer, length, &virname, acdata ? (acdata[0]): (&mdata), offset, NULL, ftype, NULL, AC_SCAN_VIR, NULL, *ctx->fmap, NULL, ctx);
+	ret = matcher_run(troot, buffer, length, &virname, acdata ? (acdata[0]): (&mdata), offset, NULL, ftype, NULL, AC_SCAN_VIR, PCRE_SCAN_BUFF, NULL, *ctx->fmap, NULL, NULL, ctx);
 
 	if(!acdata)
 	    cli_ac_freedata(&mdata);
@@ -219,7 +291,7 @@ int cli_scanbuff(const unsigned char *buffer, uint32_t length, uint32_t offset,
     if(!acdata && (ret = cli_ac_initdata(&mdata, groot->ac_partsigs, groot->ac_lsigs, groot->ac_reloff_num, CLI_DEFAULT_AC_TRACKLEN)))
 	return ret;
 
-    ret = matcher_run(groot, buffer, length, &virname, acdata ? (acdata[1]): (&mdata), offset, NULL, ftype, NULL, AC_SCAN_VIR, NULL, *ctx->fmap, NULL, ctx);
+    ret = matcher_run(groot, buffer, length, &virname, acdata ? (acdata[1]): (&mdata), offset, NULL, ftype, NULL, AC_SCAN_VIR, PCRE_SCAN_BUFF, NULL, *ctx->fmap, NULL, NULL, ctx);
 
     if(!acdata)
 	cli_ac_freedata(&mdata);
@@ -430,6 +502,11 @@ void cli_targetinfo(struct cli_target_info *info, unsigned int target, fmap_t *m
 }
 
 int cli_checkfp(unsigned char *digest, size_t size, cli_ctx *ctx)
+{
+    return cli_checkfp_virus(digest, size, ctx, NULL);
+}
+
+int cli_checkfp_virus(unsigned char *digest, size_t size, cli_ctx *ctx, const char * vname)
 {
     char md5[33];
     unsigned int i;
@@ -455,11 +532,11 @@ int cli_checkfp(unsigned char *digest, size_t size, cli_ctx *ctx)
             sprintf(md5 + i * 2, "%02x", digest[i]);
         md5[32] = 0;
         cli_dbgmsg("FP SIGNATURE: %s:%u:%s\n", md5, (unsigned int) size,
-               cli_get_last_virus(ctx) ? cli_get_last_virus(ctx) : "Name");
+                   vname ? vname : "Name");
     }
 
-    if(cli_get_last_virus(ctx))
-        do_dsig_check = strncmp("W32S.", cli_get_last_virus(ctx), 5);
+    if(vname)
+        do_dsig_check = strncmp("W32S.", vname, 5);
 
     map = *ctx->fmap;
     have_sha1 = cli_hm_have_size(ctx->engine->hm_fp, CLI_HASH_SHA1, size)
@@ -516,7 +593,7 @@ int cli_checkfp(unsigned char *digest, size_t size, cli_ctx *ctx)
             for(i=0; i<SHA1_HASH_SIZE; i++)
                 sprintf((char *)shash1+i*2, "%02x", shash1[SHA1_HASH_SIZE+i]);
 
-            cli_errmsg("COLLECT:%s:%s:%u:%s:%s\n", shash256, shash1, size, cli_get_last_virus(ctx), ctx->entry_filename);
+            cli_errmsg("COLLECT:%s:%s:%u:%s:%s\n", shash256, shash1, size, vname?vname:"noname", ctx->entry_filename);
         } else
             cli_errmsg("can't compute sha\n!");
 
@@ -544,10 +621,10 @@ int cli_checkfp(unsigned char *digest, size_t size, cli_ctx *ctx)
     }
 
     if (ctx->engine->cb_hash)
-        ctx->engine->cb_hash(fmap_fd(*ctx->fmap), size, (const unsigned char *)md5, cli_get_last_virus(ctx), ctx->cb_ctx);
+        ctx->engine->cb_hash(fmap_fd(*ctx->fmap), size, (const unsigned char *)md5, vname?vname:"noname", ctx->cb_ctx);
 
     if (ctx->engine->cb_stats_add_sample)
-        ctx->engine->cb_stats_add_sample(cli_get_last_virus(ctx), digest, size, &sections, ctx->engine->stats_data);
+        ctx->engine->cb_stats_add_sample(vname?vname:"noname", digest, size, &sections, ctx->engine->stats_data);
 
     if (sections.sections)
         free(sections.sections);
@@ -633,88 +710,144 @@ int cli_scandesc(int desc, cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struc
     return ret;
 }
 
-int cli_lsig_eval(cli_ctx *ctx, struct cli_matcher *root, struct cli_ac_data *acdata, struct cli_target_info *target_info, const char *hash)
+static int intermediates_eval(cli_ctx *ctx, struct cli_ac_lsig *ac_lsig)
 {
-	unsigned int i, evalcnt;
-	uint64_t evalids;
-	fmap_t *map = *ctx->fmap;
-	unsigned int viruses_found = 0;
+    uint32_t i, icnt = ac_lsig->tdb.intermediates[0];
+    int32_t j = -1;
+
+    if (ctx->recursion < icnt)
+        return 0;
+
+    for (i = icnt; i > 0; i--) {
+        if (ac_lsig->tdb.intermediates[i] == CL_TYPE_ANY)
+            continue;
+        if (ac_lsig->tdb.intermediates[i] != cli_get_container_intermediate(ctx, j--))
+            return 0;
+    }
+    return 1;
+}
+
+static int lsig_eval(cli_ctx *ctx, struct cli_matcher *root, struct cli_ac_data *acdata, struct cli_target_info *target_info, const char *hash, uint32_t lsid)
+{
+    unsigned evalcnt = 0;
+    uint64_t evalids = 0;
+    fmap_t *map = *ctx->fmap;
+    struct cli_ac_lsig *ac_lsig = root->ac_lsigtable[lsid];
+    char * exp = ac_lsig->u.logic;
+    char* exp_end = exp + strlen(exp);
+    int rc;
+
+    rc = cli_ac_chkmacro(root, acdata, lsid);
+    if (rc != CL_SUCCESS)
+        return rc;
+    if (cli_ac_chklsig(exp, exp_end, acdata->lsigcnt[lsid], &evalcnt, &evalids, 0) == 1) {
+        if(ac_lsig->tdb.container && ac_lsig->tdb.container[0] != cli_get_container(ctx, -1))
+            return CL_CLEAN;
+        if(ac_lsig->tdb.intermediates && !intermediates_eval(ctx, ac_lsig))
+            return CL_CLEAN;
+        if(ac_lsig->tdb.filesize && (ac_lsig->tdb.filesize[0] > map->len || ac_lsig->tdb.filesize[1] < map->len))
+            return CL_CLEAN;
+
+        if(ac_lsig->tdb.ep || ac_lsig->tdb.nos) {
+            if(!target_info || target_info->status != 1)
+                return CL_CLEAN;
+            if(ac_lsig->tdb.ep && (ac_lsig->tdb.ep[0] > target_info->exeinfo.ep || ac_lsig->tdb.ep[1] < target_info->exeinfo.ep))
+                return CL_CLEAN;
+            if(ac_lsig->tdb.nos && (ac_lsig->tdb.nos[0] > target_info->exeinfo.nsections || ac_lsig->tdb.nos[1] < target_info->exeinfo.nsections))
+                return CL_CLEAN;
+        }
+
+        if(hash && ac_lsig->tdb.handlertype) {
+            if(memcmp(ctx->handlertype_hash, hash, 16)) {
+                ctx->recursion++;
+                memcpy(ctx->handlertype_hash, hash, 16);
+                if(cli_magic_scandesc_type(ctx, ac_lsig->tdb.handlertype[0]) == CL_VIRUS) {
+                    ctx->recursion--;
+                    return CL_VIRUS;
+                }
+                ctx->recursion--;
+                return CL_CLEAN;
+            }
+        }
+        
+        if(ac_lsig->tdb.icongrp1 || ac_lsig->tdb.icongrp2) {
+            if(!target_info || target_info->status != 1)
+                return CL_CLEAN;
+            if(matchicon(ctx, &target_info->exeinfo, ac_lsig->tdb.icongrp1, ac_lsig->tdb.icongrp2) == CL_VIRUS) {
+                if(!ac_lsig->bc_idx) {
+                    rc = cli_append_virus(ctx, ac_lsig->virname);
+                    if (rc != CL_CLEAN)
+                        return rc;
+                } else if(cli_bytecode_runlsig(ctx, target_info, &ctx->engine->bcs, ac_lsig->bc_idx, acdata->lsigcnt[lsid], acdata->lsigsuboff_first[lsid], map) == CL_VIRUS) {
+                    return CL_VIRUS;
+                }
+            }
+            return CL_CLEAN;
+        }
+        if(!ac_lsig->bc_idx) {
+            rc = cli_append_virus(ctx, ac_lsig->virname);
+            if (rc != CL_CLEAN)
+                return rc;
+        }
+        if(cli_bytecode_runlsig(ctx, target_info, &ctx->engine->bcs, ac_lsig->bc_idx, acdata->lsigcnt[lsid], acdata->lsigsuboff_first[lsid], map) == CL_VIRUS) {
+            return CL_VIRUS;
+        }
+    }
+    
+    return CL_CLEAN;
+}
+
+#ifdef HAVE_YARA
+static int yara_eval(cli_ctx *ctx, struct cli_matcher *root, struct cli_ac_data *acdata, struct cli_target_info *target_info, const char *hash, uint32_t lsid)
+{
+    struct cli_ac_lsig *ac_lsig = root->ac_lsigtable[lsid];
+    int rc;
+    YR_SCAN_CONTEXT context;
+
+    (void)hash;
+ 
+    memset(&context, 0, sizeof(YR_SCAN_CONTEXT));
+    context.fmap = *ctx->fmap;
+    context.file_size = (*ctx->fmap)->len;
+    if (target_info != NULL) {
+        if (target_info->status == 1)   
+            context.entry_point = target_info->exeinfo.ep;
+    }
+
+    rc = yr_execute_code(ac_lsig, acdata, &context, 0, 0);
+
+    if (rc == CL_VIRUS) {
+        if (ac_lsig->flag & CLI_LSIG_FLAG_PRIVATE) {
+            rc = CL_CLEAN;
+        } else {
+            rc = cli_append_virus(ctx, ac_lsig->virname);
+        }
+    }
+    return rc;
+}
+#endif
+
+int cli_exp_eval(cli_ctx *ctx, struct cli_matcher *root, struct cli_ac_data *acdata, struct cli_target_info *target_info, const char *hash)
+{
+    uint8_t viruses_found = 0;
+    uint32_t i;
+    int32_t rc = CL_SUCCESS;
 
     for(i = 0; i < root->ac_lsigs; i++) {
-	evalcnt = 0;
-	evalids = 0;
-	cli_ac_chkmacro(root, acdata, i);
-	if(cli_ac_chklsig(root->ac_lsigtable[i]->logic, root->ac_lsigtable[i]->logic + strlen(root->ac_lsigtable[i]->logic), acdata->lsigcnt[i], &evalcnt, &evalids, 0) == 1) {
-	    if(root->ac_lsigtable[i]->tdb.container && root->ac_lsigtable[i]->tdb.container[0] != ctx->container_type)
-		continue;
-	    if(root->ac_lsigtable[i]->tdb.filesize && (root->ac_lsigtable[i]->tdb.filesize[0] > map->len || root->ac_lsigtable[i]->tdb.filesize[1] < map->len))
-		continue;
-
-	    if(root->ac_lsigtable[i]->tdb.ep || root->ac_lsigtable[i]->tdb.nos) {
-		if(!target_info || target_info->status != 1)
-		    continue;
-		if(root->ac_lsigtable[i]->tdb.ep && (root->ac_lsigtable[i]->tdb.ep[0] > target_info->exeinfo.ep || root->ac_lsigtable[i]->tdb.ep[1] < target_info->exeinfo.ep))
-		    continue;
-		if(root->ac_lsigtable[i]->tdb.nos && (root->ac_lsigtable[i]->tdb.nos[0] > target_info->exeinfo.nsections || root->ac_lsigtable[i]->tdb.nos[1] < target_info->exeinfo.nsections))
-		    continue;
-	    }
-
-	    if(hash && root->ac_lsigtable[i]->tdb.handlertype) {
-		if(memcmp(ctx->handlertype_hash, hash, 16)) {
-		    ctx->recursion++;
-		    memcpy(ctx->handlertype_hash, hash, 16);
-		    if(cli_magic_scandesc_type(ctx, root->ac_lsigtable[i]->tdb.handlertype[0]) == CL_VIRUS) {
-			ctx->recursion--;
-			if (SCAN_ALL) {
-			    viruses_found++;
-			    continue;
-			}
-			return CL_VIRUS;
-		    }
-		    ctx->recursion--;
-		    continue;
-		}
-	    }
-
-	    if(root->ac_lsigtable[i]->tdb.icongrp1 || root->ac_lsigtable[i]->tdb.icongrp2) {
-		if(!target_info || target_info->status != 1)
-		    continue;
-		if(matchicon(ctx, &target_info->exeinfo, root->ac_lsigtable[i]->tdb.icongrp1, root->ac_lsigtable[i]->tdb.icongrp2) == CL_VIRUS) {
-		    if(!root->ac_lsigtable[i]->bc_idx) {
-			cli_append_virus(ctx, root->ac_lsigtable[i]->virname);
-			if (SCAN_ALL) {
-                            viruses_found++;
-                            continue;
-                        }
-			return CL_VIRUS;
-		    } else if(cli_bytecode_runlsig(ctx, target_info, &ctx->engine->bcs, root->ac_lsigtable[i]->bc_idx, acdata->lsigcnt[i], acdata->lsigsuboff_first[i], map) == CL_VIRUS) {
-			if (SCAN_ALL) {
-                            viruses_found++;
-                            continue;
-                        }
-			return CL_VIRUS;
-		    }
-		}
-		continue;
-	    }
-	    if(!root->ac_lsigtable[i]->bc_idx) {
-		cli_append_virus(ctx, root->ac_lsigtable[i]->virname);
-		if (SCAN_ALL) {
-		    viruses_found++;
-		    continue;
-		}
- 		return CL_VIRUS;
-	    }
-	    if(cli_bytecode_runlsig(ctx, target_info, &ctx->engine->bcs, root->ac_lsigtable[i]->bc_idx, acdata->lsigcnt[i], acdata->lsigsuboff_first[i], map) == CL_VIRUS) {
-		if (SCAN_ALL) {
-		    viruses_found++;
-		    continue;
-		}
- 		return CL_VIRUS;
-	    }
-	}
+        if (root->ac_lsigtable[i]->type == CLI_LSIG_NORMAL)
+            rc = lsig_eval(ctx, root, acdata, target_info, hash, i);
+#ifdef HAVE_YARA
+        else if (root->ac_lsigtable[i]->type == CLI_YARA_NORMAL || root->ac_lsigtable[i]->type == CLI_YARA_OFFSET)
+            rc = yara_eval(ctx, root, acdata, target_info, hash, i);
+#endif
+        if (rc == CL_VIRUS) {
+            viruses_found = 1;
+            if (SCAN_ALL)
+                continue;
+            break;
+        }
     }
-    if (SCAN_ALL && viruses_found)
+    if (viruses_found)
 	return CL_VIRUS;
     return CL_CLEAN;
 }
@@ -722,11 +855,12 @@ int cli_lsig_eval(cli_ctx *ctx, struct cli_matcher *root, struct cli_ac_data *ac
 int cli_fmap_scandesc(cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli_matched_type **ftoffset, unsigned int acmode, struct cli_ac_result **acres, unsigned char *refhash)
 {
     const unsigned char *buff;
-    int ret = CL_CLEAN, type = CL_CLEAN, bytes, compute_hash[CLI_HASH_AVAIL_TYPES];
+    int ret = CL_CLEAN, type = CL_CLEAN, compute_hash[CLI_HASH_AVAIL_TYPES];
     unsigned int i = 0, j = 0, bm_offmode = 0;
-    uint32_t maxpatlen, offset = 0;
+    uint32_t maxpatlen, bytes, offset = 0;
     struct cli_ac_data gdata, tdata;
     struct cli_bm_off toff;
+    struct cli_pcre_off gpoff, tpoff;
     unsigned char digest[CLI_HASH_AVAIL_TYPES][32];
     struct cli_matcher *groot = NULL, *troot = NULL;
     struct cli_target_info info;
@@ -802,12 +936,26 @@ int cli_fmap_scandesc(cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli
             cl_hash_destroy(sha256ctx);
             return ret;
         }
+        if((ret = cli_pcre_recaloff(groot, &gpoff, &info, ctx))) {
+            cli_ac_freedata(&gdata);
+            if(info.exeinfo.section)
+                free(info.exeinfo.section);
+
+            cli_hashset_destroy(&info.exeinfo.vinfo);
+            cl_hash_destroy(md5ctx);
+            cl_hash_destroy(sha1ctx);
+            cl_hash_destroy(sha256ctx);
+            return ret;
+
+        }
     }
 
     if(troot) {
         if((ret = cli_ac_initdata(&tdata, troot->ac_partsigs, troot->ac_lsigs, troot->ac_reloff_num, CLI_DEFAULT_AC_TRACKLEN)) || (ret = cli_ac_caloff(troot, &tdata, &info))) {
-            if(!ftonly)
+            if(!ftonly) {
                 cli_ac_freedata(&gdata);
+                cli_pcre_freeoff(&gpoff);
+            }
             if(info.exeinfo.section)
                 free(info.exeinfo.section);
 
@@ -820,8 +968,10 @@ int cli_fmap_scandesc(cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli
         if(troot->bm_offmode) {
             if(map->len >= CLI_DEFAULT_BM_OFFMODE_FSIZE) {
                 if((ret = cli_bm_initoff(troot, &toff, &info))) {
-                    if(!ftonly)
+                    if(!ftonly) {
                         cli_ac_freedata(&gdata);
+                        cli_pcre_freeoff(&gpoff);
+                    }
 
                     cli_ac_freedata(&tdata);
                     if(info.exeinfo.section)
@@ -837,6 +987,24 @@ int cli_fmap_scandesc(cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli
                 bm_offmode = 1;
             }
         }
+        if ((ret = cli_pcre_recaloff(troot, &tpoff, &info, ctx))) {
+            if(!ftonly) {
+                cli_ac_freedata(&gdata);
+                cli_pcre_freeoff(&gpoff);
+            }
+
+            cli_ac_freedata(&tdata);
+            if(bm_offmode)
+                cli_bm_freeoff(&toff);
+            if(info.exeinfo.section)
+                free(info.exeinfo.section);
+
+            cli_hashset_destroy(&info.exeinfo.vinfo);
+            cl_hash_destroy(md5ctx);
+            cl_hash_destroy(sha1ctx);
+            cl_hash_destroy(sha256ctx);
+            return ret;
+        }
     }
 
     hdb = ctx->engine->hm_hdb;
@@ -844,7 +1012,8 @@ int cli_fmap_scandesc(cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli
 
     if(!ftonly && hdb) {
         if(!refhash) {
-            if(cli_hm_have_size(hdb, CLI_HASH_MD5, map->len) || cli_hm_have_size(fp, CLI_HASH_MD5, map->len)) {
+            if(cli_hm_have_size(hdb, CLI_HASH_MD5, map->len) || cli_hm_have_size(fp, CLI_HASH_MD5, map->len)
+               || cli_hm_have_wild(hdb, CLI_HASH_MD5) || cli_hm_have_wild(fp, CLI_HASH_MD5)) {
                 compute_hash[CLI_HASH_MD5] = 1;
             } else {
                 compute_hash[CLI_HASH_MD5] = 0;
@@ -878,19 +1047,22 @@ int cli_fmap_scandesc(cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli
 
         if(troot) {
                 virname = NULL;
-                ret = matcher_run(troot, buff, bytes, &virname, &tdata, offset, &info, ftype, ftoffset, acmode, acres, map, bm_offmode ? &toff : NULL, ctx);
+                ret = matcher_run(troot, buff, bytes, &virname, &tdata, offset, &info, ftype, ftoffset, acmode, PCRE_SCAN_FMAP, acres, map, bm_offmode ? &toff : NULL, &tpoff, ctx);
 
             if (virname) {
                 /* virname already appended by matcher_run */
                 viruses_found = 1;
             }
             if((ret == CL_VIRUS && !SCAN_ALL) || ret == CL_EMEM) {
-                if(!ftonly)
+                if(!ftonly) {
                     cli_ac_freedata(&gdata);
+                    cli_pcre_freeoff(&gpoff);
+                }
 
                 cli_ac_freedata(&tdata);
                 if(bm_offmode)
                     cli_bm_freeoff(&toff);
+                cli_pcre_freeoff(&tpoff);
 
                 if(info.exeinfo.section)
                     free(info.exeinfo.section);
@@ -905,7 +1077,7 @@ int cli_fmap_scandesc(cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli
 
         if(!ftonly) {
             virname = NULL;
-            ret = matcher_run(groot, buff, bytes, &virname, &gdata, offset, &info, ftype, ftoffset, acmode, acres, map, NULL, ctx);
+            ret = matcher_run(groot, buff, bytes, &virname, &gdata, offset, &info, ftype, ftoffset, acmode, PCRE_SCAN_FMAP, acres, map, NULL, &gpoff, ctx);
 
             if (virname) {
                 /* virname already appended by matcher_run */
@@ -913,10 +1085,12 @@ int cli_fmap_scandesc(cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli
             }
             if((ret == CL_VIRUS && !SCAN_ALL) || ret == CL_EMEM) {
                 cli_ac_freedata(&gdata);
+                cli_pcre_freeoff(&gpoff);
                 if(troot) {
                     cli_ac_freedata(&tdata);
                     if(bm_offmode)
                         cli_bm_freeoff(&toff);
+                    cli_pcre_freeoff(&tpoff);
                 }
 
                 if(info.exeinfo.section)
@@ -1010,19 +1184,18 @@ int cli_fmap_scandesc(cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli
             /* If matched size-based hash ... */
             if (found % 2) {
                 viruses_found = 1;
-                cli_append_virus(ctx, virname);
-                if (!SCAN_ALL)
+                ret = cli_append_virus(ctx, virname);
+                if (!SCAN_ALL || ret != CL_CLEAN)
                     break;
                 virname = NULL;
             }
             /* If matched size-agnostic hash ... */
             if (found > 1) {
                 viruses_found = 1;
-                cli_append_virus(ctx, virname_w);
-
-                if (!SCAN_ALL)
+                ret = cli_append_virus(ctx, virname_w);
+                if (!SCAN_ALL || ret != CL_CLEAN)
                     break;
-            }
+             }
         }
     }
 
@@ -1032,19 +1205,21 @@ int cli_fmap_scandesc(cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli
 
     if(troot) {
         if(ret != CL_VIRUS || SCAN_ALL)
-            ret = cli_lsig_eval(ctx, troot, &tdata, &info, (const char *)refhash);
+            ret = cli_exp_eval(ctx, troot, &tdata, &info, (const char *)refhash);
         if (ret == CL_VIRUS)
             viruses_found++;
 
         cli_ac_freedata(&tdata);
         if(bm_offmode)
             cli_bm_freeoff(&toff);
+        cli_pcre_freeoff(&tpoff);
     }
 
     if(groot) {
         if(ret != CL_VIRUS || SCAN_ALL)
-            ret = cli_lsig_eval(ctx, groot, &gdata, &info, (const char *)refhash);
+            ret = cli_exp_eval(ctx, groot, &gdata, &info, (const char *)refhash);
         cli_ac_freedata(&gdata);
+        cli_pcre_freeoff(&gpoff);
     }
 
     if(info.exeinfo.section)
@@ -1064,25 +1239,27 @@ int cli_matchmeta(cli_ctx *ctx, const char *fname, size_t fsizec, size_t fsizer,
 {
 	const struct cli_cdb *cdb;
 	unsigned int viruses_found = 0;
+        int ret = CL_CLEAN;
 
-    cli_dbgmsg("CDBNAME:%s:%lu:%s:%lu:%lu:%d:%u:%u:%p\n",
-	       cli_ftname(ctx->container_type), fsizec, fname, fsizec, fsizer, encrypted, filepos, res1, res2);
+    cli_dbgmsg("CDBNAME:%s:%llu:%s:%llu:%llu:%d:%u:%u:%p\n",
+	       cli_ftname(cli_get_container(ctx, -1)), (long long unsigned)fsizec, fname, (long long unsigned)fsizec, (long long unsigned)fsizer,
+	       encrypted, filepos, res1, res2);
 
     if (ctx->engine && ctx->engine->cb_meta)
-	if (ctx->engine->cb_meta(cli_ftname(ctx->container_type), fsizec, fname, fsizer, encrypted, filepos, ctx->cb_ctx) == CL_VIRUS) {
+	if (ctx->engine->cb_meta(cli_ftname(cli_get_container(ctx, -1)), fsizec, fname, fsizer, encrypted, filepos, ctx->cb_ctx) == CL_VIRUS) {
 	    cli_dbgmsg("inner file blacklisted by callback: %s\n", fname);
 
-	    cli_append_virus(ctx, "Detected.By.Callback");
+	    ret = cli_append_virus(ctx, "Detected.By.Callback");
 	    viruses_found++;
-	    if(!SCAN_ALL)
-		return CL_VIRUS;
+	    if(!SCAN_ALL || ret != CL_CLEAN)
+		return ret;
 	}
 
     if(!ctx->engine || !(cdb = ctx->engine->cdb))
 	return CL_CLEAN;
 
     do {
-	if(cdb->ctype != CL_TYPE_ANY && cdb->ctype != ctx->container_type)
+	if(cdb->ctype != CL_TYPE_ANY && cdb->ctype != cli_get_container(ctx, -1))
 	    continue;
 
 	if(cdb->encrypted != 2 && cdb->encrypted != encrypted)
@@ -1091,16 +1268,17 @@ int cli_matchmeta(cli_ctx *ctx, const char *fname, size_t fsizec, size_t fsizer,
 	if(cdb->res1 && (cdb->ctype == CL_TYPE_ZIP || cdb->ctype == CL_TYPE_RAR) && cdb->res1 != res1)
 	    continue;
 
-#define CDBRANGE(field, val)						    \
-	if(field[0] != CLI_OFF_ANY) {					    \
-	    if(field[0] == field[1] && field[0] != val)			    \
-		continue;						    \
-	    else if(field[0] != field[1] && ((field[0] && field[0] > val) ||\
-	      (field[1] && field[1] < val)))				    \
-		continue;						    \
-	}
+    #define CDBRANGE(field, val)                                              \
+        if (field[0] != CLI_OFF_ANY)                                          \
+        {                                                                     \
+            if (field[0] == field[1] && field[0] != val)                      \
+                continue;                                                     \
+            else if (field[0] != field[1] && ((field[0] && field[0] > val) || \
+                                            (field[1] && field[1] < val)))    \
+                continue;                                                     \
+        }
 
-	CDBRANGE(cdb->csize, ctx->container_size);
+    CDBRANGE(cdb->csize, cli_get_container_size(ctx, -1));
 	CDBRANGE(cdb->fsizec, fsizec);
 	CDBRANGE(cdb->fsizer, fsizer);
 	CDBRANGE(cdb->filepos, filepos);
@@ -1108,10 +1286,10 @@ int cli_matchmeta(cli_ctx *ctx, const char *fname, size_t fsizec, size_t fsizer,
 	if(cdb->name.re_magic && (!fname || cli_regexec(&cdb->name, fname, 0, NULL, 0) == REG_NOMATCH))
 	    continue;
 
-	cli_append_virus(ctx, cdb->virname);
+	ret = cli_append_virus(ctx, cdb->virname);
 	viruses_found++;
-	if(!SCAN_ALL)
-	    return CL_VIRUS;
+	if(!SCAN_ALL || ret != CL_CLEAN)
+	    return ret;
 
     } while((cdb = cdb->next));
 
