@@ -1,7 +1,7 @@
 /*
- * Extract component parts of MS XML files (e.g. MS Office 2003 XML Documents)
+ * Extract component parts of various MS XML files (e.g. MS Office 2003 XML Documents)
  *
- * Copyright (C) 2015, 2018 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ * Copyright (C) 2015, 2017 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  * Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  * Authors: Kevin Lin
@@ -62,10 +62,23 @@
         }                                                               \
     } while(0)
 
+#define track_json(mxctx) (mxctx->ictx->flags & MSXML_FLAG_JSON)
+
+struct msxml_ictx {
+    cli_ctx *ctx;
+    uint32_t flags;
+    const struct key_entry *keys;
+    size_t num_keys;
+
+#if HAVE_JSON
+    json_object *root;
+    int toval;
+#endif
+};
 
 struct key_entry blank_key = { NULL, NULL, 0 };
 
-static const struct key_entry *msxml_check_key(struct msxml_ctx *mxctx, const xmlChar *key, size_t keylen)
+static const struct key_entry *msxml_check_key(struct msxml_ictx *ictx, const xmlChar *key, size_t keylen)
 {
     unsigned i;
 
@@ -74,9 +87,9 @@ static const struct key_entry *msxml_check_key(struct msxml_ctx *mxctx, const xm
         return &blank_key;
     }
 
-    for (i = 0; i < mxctx->num_keys; ++i) {
-        if (keylen == strlen(mxctx->keys[i].key) && !strncasecmp((char *)key, mxctx->keys[i].key, keylen)) {
-            return &mxctx->keys[i];
+    for (i = 0; i < ictx->num_keys; ++i) {
+        if (keylen == strlen(ictx->keys[i].key) && !strncasecmp((char *)key, ictx->keys[i].key, keylen)) {
+            return &ictx->keys[i];
         }
     }
 
@@ -91,11 +104,11 @@ static void msxml_error_handler(void* arg, const char* msg, xmlParserSeverities
     switch (severity) {
     case XML_PARSER_SEVERITY_WARNING:
     case XML_PARSER_SEVERITY_VALIDITY_WARNING:
-        cli_warnmsg("%s:%d: parser warning : %s", (char*)URI, line, msg);
+        cli_dbgmsg("%s:%d: parser warning : %s", (char*)URI, line, msg);
         break;
     case XML_PARSER_SEVERITY_ERROR:
     case XML_PARSER_SEVERITY_VALIDITY_ERROR:
-        cli_warnmsg("%s:%d: parser error : %s", (char*)URI, line, msg);
+        cli_dbgmsg("%s:%d: parser error : %s", (char*)URI, line, msg);
         break;
     default:
         cli_dbgmsg("%s:%d: unknown severity : %s", (char*)URI, line, msg);
@@ -164,8 +177,9 @@ static int msxml_parse_element(struct msxml_ctx *mxctx, xmlTextReaderPtr reader,
     const struct key_entry *keyinfo;
     struct attrib_entry attribs[MAX_ATTRIBS];
     int ret, virus = 0, state, node_type, endtag = 0, num_attribs = 0;
-    cli_ctx *ctx = mxctx->ctx;
+    cli_ctx *ctx = mxctx->ictx->ctx;
 #if HAVE_JSON
+    json_object *root = mxctx->ictx->root;
     json_object *parent = (json_object *)jptr;
     json_object *thisjobj = NULL;
 #else
@@ -180,8 +194,8 @@ static int msxml_parse_element(struct msxml_ctx *mxctx, xmlTextReaderPtr reader,
         cli_dbgmsg("msxml_parse_element: reached msxml json recursion limit\n");
 
 #if HAVE_JSON
-        if (mxctx->mode) {
-            int tmp = cli_json_parse_error(mxctx->root, "MSXML_RECURSIVE_LIMIT");
+        if (track_json(mxctx)) {
+            int tmp = cli_json_parse_error(root, "MSXML_RECURSIVE_LIMIT");
             if (tmp != CL_SUCCESS)
                 return tmp;
         }
@@ -211,8 +225,8 @@ static int msxml_parse_element(struct msxml_ctx *mxctx, xmlTextReaderPtr reader,
         if (!element_name) {
             cli_dbgmsg("msxml_parse_element: element tag node nameless\n");
 #if HAVE_JSON
-            if (mxctx->mode) {
-                int tmp = cli_json_parse_error(mxctx->root, "MSXML_NAMELESS_ELEMENT");
+            if (track_json(mxctx)) {
+                int tmp = cli_json_parse_error(root, "MSXML_NAMELESS_ELEMENT");
                 if (tmp != CL_SUCCESS)
                     return tmp;
             }
@@ -221,7 +235,7 @@ static int msxml_parse_element(struct msxml_ctx *mxctx, xmlTextReaderPtr reader,
         }
 
         /* determine if the element is interesting */
-        keyinfo = msxml_check_key(mxctx, element_name, xmlStrlen(element_name));
+        keyinfo = msxml_check_key(mxctx->ictx, element_name, xmlStrlen(element_name));
 
         cli_msxmlmsg("key:  %s\n", keyinfo->key);
         cli_msxmlmsg("name: %s\n", keyinfo->name);
@@ -237,9 +251,9 @@ static int msxml_parse_element(struct msxml_ctx *mxctx, xmlTextReaderPtr reader,
         }
 
 #if HAVE_JSON
-        if (mxctx->mode && (keyinfo->type & MSXML_JSON_TRACK)) {
+        if (track_json(mxctx) && (keyinfo->type & MSXML_JSON_TRACK)) {
             if (keyinfo->type & MSXML_JSON_ROOT)
-                thisjobj = cli_jsonobj(mxctx->root, keyinfo->name);
+                thisjobj = cli_jsonobj(root, keyinfo->name);
             else if (keyinfo->type & MSXML_JSON_WRKPTR)
                 thisjobj = cli_jsonobj(parent, keyinfo->name);
 
@@ -294,7 +308,7 @@ static int msxml_parse_element(struct msxml_ctx *mxctx, xmlTextReaderPtr reader,
                         value = xmlTextReaderConstValue(reader);
 
                         cli_msxmlmsg("\t%s: %s\n", name, value);
-                        cli_jsonstr(attributes, name, (const char *)value);
+                        cli_jsonstr(attributes, (char*)name, (const char *)value);
                     }
                 }
                 else if (state == -1)
@@ -353,7 +367,7 @@ static int msxml_parse_element(struct msxml_ctx *mxctx, xmlTextReaderPtr reader,
 
         while (!endtag) {
 #if HAVE_JSON
-            if (mxctx->mode && (cli_json_timeout_cycle_check(mxctx->ctx, &(mxctx->toval)) != CL_SUCCESS))
+            if (track_json(mxctx) && (cli_json_timeout_cycle_check(ctx, &(mxctx->ictx->toval)) != CL_SUCCESS))
                 return CL_ETIMEOUT;
 #endif
 
@@ -402,12 +416,15 @@ static int msxml_parse_element(struct msxml_ctx *mxctx, xmlTextReaderPtr reader,
 
                     if (cli_writen(of, (char *)node_value, vlen) != vlen) {
                         close(of);
+                        if (!(ctx->engine->keeptmp))
+                            cli_unlink(tempfile);
+                        free(tempfile);
                         return CL_EWRITE;
                     }
 
                     cli_dbgmsg("msxml_parse_element: extracted binary data to %s\n", tempfile);
 
-                    ret = mxctx->scan_cb(of, ctx, num_attribs, attribs);
+                    ret = mxctx->scan_cb(of, ctx, num_attribs, attribs, mxctx->scan_data);
                     close(of);
                     if (!(ctx->engine->keeptmp))
                         cli_unlink(tempfile);
@@ -445,7 +462,7 @@ static int msxml_parse_element(struct msxml_ctx *mxctx, xmlTextReaderPtr reader,
                     if(cli_writen(of, decoded, decodedlen) != (int)decodedlen) {
                         free(decoded);
                         close(of);
-                        if (ctx && !(ctx->engine->keeptmp))
+                        if (!(ctx->engine->keeptmp))
                             cli_unlink(tempfile);
                         free(tempfile);
                         return CL_EWRITE;
@@ -471,6 +488,31 @@ static int msxml_parse_element(struct msxml_ctx *mxctx, xmlTextReaderPtr reader,
                 check_state(state);
                 break;
 
+            case XML_READER_TYPE_COMMENT:
+                node_value = xmlTextReaderConstValue(reader);
+
+                cli_msxmlmsg("COMMENT: %s\n", node_value);
+
+                /* callback-based scanning mechanism for comments (used by MHTML) */
+                if ((keyinfo->type & MSXML_COMMENT_CB) && mxctx->comment_cb) {
+#if HAVE_JSON
+                    ret = mxctx->comment_cb((const char *)node_value, ctx, thisjobj, mxctx->comment_data);
+#else
+                    ret = mxctx->comment_cb((const char *)node_value, ctx, NULL, mxctx->comment_data);
+#endif
+                    if (ret != CL_SUCCESS && (ret != CL_VIRUS || (!SCAN_ALL && ret == CL_VIRUS))) {
+                        return ret;
+                    } else if (SCAN_ALL && ret == CL_VIRUS) {
+                        virus = 1;
+                    }
+
+                }
+
+                /* advance to next node */
+                state = xmlTextReaderRead(reader);
+                check_state(state);
+                break;
+
             case XML_READER_TYPE_SIGNIFICANT_WHITESPACE:
                 /* advance to next node */
                 state = xmlTextReaderRead(reader);
@@ -503,9 +545,8 @@ static int msxml_parse_element(struct msxml_ctx *mxctx, xmlTextReaderPtr reader,
 
                 cli_dbgmsg("msxml_parse_element: unhandled xml secondary node %s [%d]: %s\n", node_name, node_type, node_value);
 
-                state = xmlTextReaderNext(reader);
+                state = xmlTextReaderRead(reader);
                 check_state(state);
-                return (virus ? CL_VIRUS : CL_SUCCESS);
             }
         }
 
@@ -526,43 +567,52 @@ static int msxml_parse_element(struct msxml_ctx *mxctx, xmlTextReaderPtr reader,
     return (virus ? CL_VIRUS : CL_SUCCESS);
 }
 
-/* reader intialization and closing handled by caller */
-int cli_msxml_parse_document(cli_ctx *ctx, xmlTextReaderPtr reader, const struct key_entry *keys, const size_t num_keys, int mode, msxml_scan_cb scan_cb)
+/* reader initialization and closing handled by caller */
+int cli_msxml_parse_document(cli_ctx *ctx, xmlTextReaderPtr reader, const struct key_entry *keys, const size_t num_keys, uint32_t flags, struct msxml_ctx *mxctx)
 {
-    struct msxml_ctx mxctx;
+    struct msxml_ctx reserve;
+    struct msxml_ictx ictx;
     int state, virus = 0, ret = CL_SUCCESS;
 
     if (!ctx)
         return CL_ENULLARG;
 
-    mxctx.ctx = ctx;
-    mxctx.scan_cb = scan_cb;
-    mxctx.keys = keys;
-    mxctx.num_keys = num_keys;
-#if HAVE_JSON
-    mxctx.mode = mode;
-    if (mode) {
-        mxctx.root = ctx->wrkproperty;
-        /* JSON Sanity Check */
-        if (!mxctx.root)
-            mxctx.mode = 0;
-        mxctx.toval = 0;
+    if (!mxctx) {
+        memset(&reserve, 0, sizeof(reserve));
+        mxctx = &reserve;
     }
-#endif
 
-    /* Error Handler */
-    xmlTextReaderSetErrorHandler(reader, NULL, NULL); /* xml default handler */
-    //xmlTextReaderSetErrorHandler(reader, msxml_error_handler, NULL);
+    ictx.ctx = ctx;
+    ictx.flags = flags;
+    ictx.keys = keys;
+    ictx.num_keys = num_keys;
+#if HAVE_JSON
+    if (flags & MSXML_FLAG_JSON) {
+        ictx.root = ctx->wrkproperty;
+        /* JSON Sanity Check */
+        if (!ictx.root)
+            ictx.flags &= ~MSXML_FLAG_JSON;
+        ictx.toval = 0;
+    }
+#else
+    ictx.flags &= ~MSXML_FLAG_JSON;
+#endif
+    mxctx->ictx = &ictx;
+
+    /* Error Handler (setting handler on tree walker causes segfault) */
+    if (!(flags & MSXML_FLAG_WALK))
+        //xmlTextReaderSetErrorHandler(reader, NULL, NULL); /* xml default handler */
+        xmlTextReaderSetErrorHandler(reader, msxml_error_handler, NULL);
 
     /* Main Processing Loop */
     while ((state = xmlTextReaderRead(reader)) == 1) {
 #if HAVE_JSON
-        if (mxctx.mode && (cli_json_timeout_cycle_check(mxctx.ctx, &(mxctx.toval)) != CL_SUCCESS))
+        if ((ictx.flags & MSXML_FLAG_JSON) && (cli_json_timeout_cycle_check(ictx.ctx, &(ictx.toval)) != CL_SUCCESS))
             return CL_ETIMEOUT;
 
-        ret = msxml_parse_element(&mxctx, reader, 0, mxctx.root);
+        ret = msxml_parse_element(mxctx, reader, 0, ictx.root);
 #else
-        ret = msxml_parse_element(&mxctx, reader, 0, NULL);
+        ret = msxml_parse_element(mxctx, reader, 0, NULL);
 #endif
         if (ret == CL_SUCCESS);
         else if (SCAN_ALL && ret == CL_VIRUS) {
@@ -582,7 +632,7 @@ int cli_msxml_parse_document(cli_ctx *ctx, xmlTextReaderPtr reader, const struct
 
 #if HAVE_JSON
     /* Parse General Error Handler */
-    if (mxctx.mode) {
+    if (ictx.flags & MSXML_FLAG_JSON) {
         int tmp = CL_SUCCESS;
 
         switch(ret) {
@@ -590,22 +640,22 @@ int cli_msxml_parse_document(cli_ctx *ctx, xmlTextReaderPtr reader, const struct
         case CL_BREAK: /* OK */
             break;
         case CL_VIRUS:
-            tmp = cli_json_parse_error(mxctx.root, "MSXML_INTR_VIRUS");
+            tmp = cli_json_parse_error(ictx.root, "MSXML_INTR_VIRUS");
             break;
         case CL_ETIMEOUT:
-            tmp = cli_json_parse_error(mxctx.root, "MSXML_INTR_TIMEOUT");
+            tmp = cli_json_parse_error(ictx.root, "MSXML_INTR_TIMEOUT");
             break;
         case CL_EPARSE:
-            tmp = cli_json_parse_error(mxctx.root, "MSXML_ERROR_XMLPARSER");
+            tmp = cli_json_parse_error(ictx.root, "MSXML_ERROR_XMLPARSER");
             break;
         case CL_EMEM:
-            tmp = cli_json_parse_error(mxctx.root, "MSXML_ERROR_OUTOFMEM");
+            tmp = cli_json_parse_error(ictx.root, "MSXML_ERROR_OUTOFMEM");
             break;
         case CL_EFORMAT:
-            tmp = cli_json_parse_error(mxctx.root, "MSXML_ERROR_MALFORMED");
+            tmp = cli_json_parse_error(ictx.root, "MSXML_ERROR_MALFORMED");
             break;
         default:
-            tmp = cli_json_parse_error(mxctx.root, "MSXML_ERROR_OTHER");
+            tmp = cli_json_parse_error(ictx.root, "MSXML_ERROR_OTHER");
             break;
         }
 
@@ -614,7 +664,7 @@ int cli_msxml_parse_document(cli_ctx *ctx, xmlTextReaderPtr reader, const struct
     }
 #endif
 
-    /* non-critical return supression */
+    /* non-critical return suppression */
     if (ret == CL_ETIMEOUT || ret == CL_BREAK)
         ret = CL_SUCCESS;
 
