@@ -1,10 +1,12 @@
 /*
- *  Extract component parts of OLE2 files (e.g. MS Office Documents)
- *
- *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2015, 2018 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2008 Sourcefire, Inc.
  *
  *  Authors: Trog
+ * 
+ *  Summary: Extract component parts of OLE2 files (e.g. MS Office Documents).
+ * 
+ *  Acknowledgements: Some ideas and algorithms were based upon OpenOffice and libgsf.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -28,6 +30,5 @@
 #include "uniq.h"
 
 int cli_ole2_extract(const char *dirname, cli_ctx *ctx, struct uniq **);
-int cli_ole2_summary_json(cli_ctx *ctx, int fd, int mode);
 
 #endif
