@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2015, 2017 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2008 Sourcefire, Inc.
  *
  *  Authors: Trog, Török Edvin
@@ -90,8 +90,7 @@ int cli_check_mydoom_log(cli_ctx *ctx)
     if ((~check) != key)
 	return CL_CLEAN;
 
-    cli_append_virus(ctx, "Heuristics.Worm.Mydoom.M.log");
-    return CL_VIRUS;
+    return cli_append_virus(ctx, "Heuristics.Worm.Mydoom.M.log");
 }
 
 static int jpeg_check_photoshop_8bim(cli_ctx *ctx, off_t *off)
@@ -253,6 +252,8 @@ static uint32_t riff_endian_convert_32(uint32_t value, int big_endian)
 
 static int riff_read_chunk(fmap_t *map, off_t *offset, int big_endian, int rec_level)
 {
+	uint32_t cache_buf;
+	char *buffer;
 	const uint32_t *buf;
 	uint32_t chunk_size;
 	off_t cur_offset = *offset;
@@ -265,7 +266,11 @@ static int riff_read_chunk(fmap_t *map, off_t *offset, int big_endian, int rec_l
 	if(!(buf = fmap_need_off_once(map, cur_offset, 4*2)))
 	    return 0;
 	cur_offset += 4*2;
-	chunk_size = riff_endian_convert_32(buf[1], big_endian);
+
+	buffer = (char *)buf;
+	memcpy (&cache_buf, buffer + sizeof (cache_buf),
+			sizeof (cache_buf));
+	chunk_size = riff_endian_convert_32(cache_buf, big_endian);
 
 	if(!memcmp(buf, "anih", 4) && chunk_size != 36)
 	    return 2;
@@ -350,7 +355,7 @@ static inline int swizz_j48(const uint16_t n[])
 void cli_detect_swizz_str(const unsigned char *str, uint32_t len, struct swizz_stats *stats, int blob)
 {
 	unsigned char stri[4096];
-        uint32_t i, j = 0;
+	size_t i, j = 0;
 	int bad = 0;
 	int lastalnum = 0;
 	uint8_t ngrams[17576];
@@ -360,7 +365,8 @@ void cli_detect_swizz_str(const unsigned char *str, uint32_t len, struct swizz_s
 	int ret;
 
 	stats->entries++;
-	for(i=0;i<len-1 && j < sizeof(stri)-2;i += 2) {
+	for (i=0; (i < (size_t)len - 1) && (j < sizeof(stri) - 2); i += 2)
+	{
 		unsigned char c = str[i];
 		if (str[i+1] || !c) {
 			bad++;
@@ -447,7 +453,7 @@ int cli_detect_swizz(struct swizz_stats *stats)
 {
 	uint32_t gn[10];
 	uint32_t all = 0;
-	unsigned i;
+	size_t i;
 	int global_swizz = CL_CLEAN;
 
 	cli_dbgmsg("cli_detect_swizz: %lu/%lu, version:%d, manifest: %d \n",
