@@ -1,8 +1,11 @@
 /*
- *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2015, 2018 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2008 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm, Nigel Horne, Török Edvin
+ * 
+ *  Acknowledgements: cli_strcasestr() contains a public domain code from:
+ *                    http://unixpapa.com/incnote/string.html
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -28,6 +31,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <limits.h>
 #ifdef HAVE_STRINGS_H
 #include <strings.h>
 #endif
@@ -460,6 +464,38 @@ const char* cli_strcasestr(const char* a, const char *b)
 }
 #endif
 
+#if !defined(HAVE_STRNLEN) || defined(HAVE_STRNI)
+size_t cli_strnlen(const char *s, size_t n)
+{
+    size_t i = 0;
+    for(; (i < n) && s[i] != '\0'; ++i);
+    return i;
+}
+#endif
+
+#if !defined(HAVE_STRNDUP) || defined(HAVE_STRNI)
+char *cli_strndup(const char *s, size_t n)
+{
+    char *alloc;
+    size_t len;
+
+    if(!s) {
+        return NULL;
+    }
+
+    len = cli_strnlen(s, n);
+    alloc = malloc(len+1);
+
+    if(!alloc) {
+        return NULL;
+    } else
+        memcpy(alloc, s, len);
+
+    alloc[len] = '\0';
+    return alloc;
+}
+#endif
+
 size_t cli_strtokenize(char *buffer, const char delim, const size_t token_count, const char **tokens)
 {
 	size_t tokens_found, i;
@@ -481,6 +517,315 @@ size_t cli_strtokenize(char *buffer, const char delim, const size_t token_count,
     return tokens_found;
 }
 
+/**
+ * @brief The strntol() function converts the string in str to a long value.
+ * Modifications made to validate the length of the string for non-null term strings.
+ *
+ * Copyright (c) 1990 The Regents of the University of California.
+ * All rights reserved.
+ *
+ * @param nptr          Pointer to start of string.
+ * @param n             Max length of buffer in bytes.
+ * @param[out] endptr   [optional] If endptr is not NULL, strtol() stores the address
+ *                      of the first invalid character in *endptr. If there were no digits
+ *                      at all, however, strtol() stores the
+ *                      original value of str in *endptr. 
+ * 	                     Nota Bene:  If the buffer is non-null terminated and the number
+ *                       comprises the entire buffer, endptr will point past the end of
+ *                       the buffer, and the caller should check if endptr >= nptr + n.
+ *                      
+ * @param int           The conversion is done according to the given base, which must be
+ *                      between 2 and 36 inclusive, or be the special value 0.
+ * @return long         The signed long value.
+ */
+static long cli_strntol(const char* nptr, size_t n, char** endptr, register int base)
+{
+    register const char* s = nptr;
+    register unsigned long acc = 0;
+    register int c;
+    register unsigned long cutoff;
+    register int neg = 0, any = 0, cutlim;
+
+    if (0 == n) {
+        goto done;
+    }
+    /*
+	 * Skip white space and pick up leading +/- sign if any.
+	 * If base is 0, allow 0x for hex and 0 for octal, else
+	 * assume decimal; if base is already 16, allow 0x.
+	 */
+    do {
+        c = *s;
+    } while (isspace(c) && (++s < nptr + n));
+
+    if (s >= nptr + n) {
+        goto done;
+    }
+
+    if (c == '-') {
+        neg = 1;
+        c = *s++;
+        if (s >= nptr + n) {
+            goto done;
+        }
+    } else if (c == '+') {
+        c = *s++;
+        if (s >= nptr + n) {
+            goto done;
+        }
+    }
+
+    if (base == 0 || base == 16) {
+        if (c == '0' && (*s == 'x' || *s == 'X')) {
+            if (s + 2 >= nptr + n) {
+                goto done;
+            }
+            c = s[1];
+            s += 2;
+            base = 16;
+        }
+    }
+
+    if (base == 0)
+        base = c == '0' ? 8 : 10;
+
+    /*
+	 * Compute the cutoff value between legal numbers and illegal
+	 * numbers.  That is the largest legal value, divided by the
+	 * base.  An input number that is greater than this value, if
+	 * followed by a legal input character, is too big.  One that
+	 * is equal to this value may be valid or not; the limit
+	 * between valid and invalid numbers is then based on the last
+	 * digit.  For instance, if the range for longs is
+	 * [-2147483648..2147483647] and the input base is 10,
+	 * cutoff will be set to 214748364 and cutlim to either
+	 * 7 (neg==0) or 8 (neg==1), meaning that if we have accumulated
+	 * a value > 214748364, or equal but the next digit is > 7 (or 8),
+	 * the number is too big, and we will return a range error.
+	 *
+	 * Set any if any `digits' consumed; make it negative to indicate
+	 * overflow.
+	 */
+    cutoff = neg ? -(unsigned long)LONG_MIN : LONG_MAX;
+    cutlim = cutoff % (unsigned long)base;
+    cutoff /= (unsigned long)base;
+    for (acc = 0, any = 0; s < nptr + n; s++) {
+        c = *s;
+
+        if (isdigit(c))
+            c -= '0';
+        else if (isalpha(c))
+            c -= isupper(c) ? 'A' - 10 : 'a' - 10;
+        else
+            break;
+        if (c >= base)
+            break;
+        if (any < 0 || acc > cutoff || (acc == cutoff && c > cutlim))
+            any = -1;
+        else {
+            any = 1;
+            acc *= base;
+            acc += c;
+        }
+    }
+    if (any < 0) {
+        acc = neg ? LONG_MIN : LONG_MAX;
+        errno = ERANGE;
+    } else if (neg)
+        acc = -acc;
+
+done:
+    if (endptr != 0)
+        *endptr = (char*)(any ? s : nptr);
+    return (acc);
+}
+
+/**
+ * @brief The strntoul() function converts the string in str to an unsigned long value.
+ * Modifications made to validate the length of the string for non-null term strings.
+ *
+ * Copyright (c) 1990 The Regents of the University of California.
+ * All rights reserved.
+ *
+ * @param nptr          Pointer to start of string.
+ * @param n             Max length of buffer in bytes.
+ * @param[out] endptr   [optional] If endptr is not NULL, strtol() stores the address
+ *                      of the first invalid character in *endptr. If there were no digits
+ *                      at all, however, strtol() stores the
+ *                      original value of str in *endptr. 
+ * 	                     Nota Bene:  If the buffer is non-null terminated and the number
+ *                       comprises the entire buffer, endptr will point past the end of
+ *                       the buffer, and the caller should check if endptr >= nptr + n.
+ *                      
+ * @param int           The conversion is done according to the given base, which must be
+ *                      between 2 and 36 inclusive, or be the special value 0.
+ * @return unsigned long The unsigned long value.
+ */
+static unsigned long
+cli_strntoul(const char* nptr, size_t n, char** endptr, register int base)
+{
+    register const char* s = nptr;
+    register unsigned long acc = 0;
+    register int c;
+    register unsigned long cutoff;
+    register int neg = 0, any = 0, cutlim;
+
+    /*
+	 * See cli_strntol for comments as to the logic used.
+	 */
+    do {
+        c = *s;
+    } while (isspace(c) && (++s < nptr + n));
+
+    if (s >= nptr + n) {
+        goto done;
+    }
+
+    if (c == '-') {
+        neg = 1;
+        c = *s++;
+        if (s >= nptr + n) {
+            goto done;
+        }
+    } else if (c == '+') {
+        c = *s++;
+        if (s >= nptr + n) {
+            goto done;
+        }
+    }
+
+    if (base == 0 || base == 16) {
+        if (c == '0' && (*s == 'x' || *s == 'X')) {
+            if (s + 2 >= nptr + n) {
+                goto done;
+            }
+            c = s[1];
+            s += 2;
+            base = 16;
+        }
+    }
+    if (base == 0)
+        base = c == '0' ? 8 : 10;
+
+    cutoff = (unsigned long)ULONG_MAX / (unsigned long)base;
+    cutlim = (unsigned long)ULONG_MAX % (unsigned long)base;
+    for (acc = 0, any = 0; s < nptr + n; s++) {
+        c = *s;
+
+        if (isdigit(c))
+            c -= '0';
+        else if (isalpha(c))
+            c -= isupper(c) ? 'A' - 10 : 'a' - 10;
+        else
+            break;
+        if (c >= base)
+            break;
+        if (any < 0 || acc > cutoff || (acc == cutoff && c > cutlim))
+            any = -1;
+        else {
+            any = 1;
+            acc *= base;
+            acc += c;
+        }
+    }
+    if (any < 0) {
+        acc = ULONG_MAX;
+        errno = ERANGE;
+    } else if (neg)
+        acc = -acc;
+
+done:
+    if (endptr != 0)
+        *endptr = (char*)(any ? s : nptr);
+    return (acc);
+}
+
+/**
+ * @brief 	cli_strntol_wrap() converts the string in str to a long value.
+ * 
+ * Wrapper for cli_strntol() that provides incentive to check for failure.
+ * 
+ * @param buf               Pointer to start of string. 
+ * @param buf_size 			Max length of buffer to convert to integer.
+ * @param fail_at_nondigit  If 1, fail out if the a non-digit character is found before the end of the buffer.
+ *                          If 0, non-digit character represents end of number and is not a failure.
+ * @param base              The conversion is done according to the given base, which must be
+ *                          between 2 and 36 inclusive, or be the special value 0.
+ * @param[out] result 	    Long integer value of ascii number.
+ * @return CL_SUCCESS       Success
+ * @return CL_EPARSE        Failure
+ */
+int cli_strntol_wrap(const char *buf, size_t buf_size, int fail_at_nondigit, int base, long *result)
+{
+    char *endptr = NULL;
+    long num;
+
+    if (buf_size == 0 || !buf || !result) {
+        /* invalid parameter */
+        return CL_EPARSE;
+    }
+    errno = 0;
+    num = cli_strntol(buf, buf_size, &endptr, base);
+    if ((num == LONG_MIN || num == LONG_MAX) && errno == ERANGE) {
+        /* under- or overflow */
+        return CL_EPARSE;
+    }
+    if (endptr == buf) {
+        /* no digits */
+        return CL_EPARSE;
+    }
+    if (fail_at_nondigit && (endptr < (buf + buf_size)) && (*endptr != '\0')) {
+        /* non-digit encountered */
+        return CL_EPARSE;
+    }
+    /* success */
+    *result = num;
+    return CL_SUCCESS;
+}
+
+/**
+ * @brief 	cli_strntoul_wrap() converts the string in str to a long value.
+ * 
+ * Wrapper for cli_strntoul() that provides incentive to check for failure.
+ * 
+ * @param buf               Pointer to start of string. 
+ * @param buf_size 			Max length of buffer to convert to integer.
+ * @param fail_at_nondigit  If 1, fail out if the a non-digit character is found before the end of the buffer.
+ *                          If 0, non-digit character represents end of number and is not a failure.
+ * @param base              The conversion is done according to the given base, which must be
+ *                          between 2 and 36 inclusive, or be the special value 0.
+ * @param[out] result 	    Unsigned long integer value of ascii number.
+ * @return CL_SUCCESS       Success
+ * @return CL_EPARSE        Failure
+ */
+int cli_strntoul_wrap(const char *buf, size_t buf_size, int fail_at_nondigit, int base, unsigned long *result)
+{
+    char *endptr = NULL;
+    long num;
+
+    if (buf_size == 0 || !buf || !result) {
+        /* invalid parameter */
+        return CL_EPARSE;
+    }
+    errno = 0;
+    num = cli_strntoul(buf, buf_size, &endptr, base);
+    if (num == ULONG_MAX && errno == ERANGE) {
+        /* under- or overflow */
+        return CL_EPARSE;
+    }
+    if (endptr == buf) {
+        /* no digits */
+        return CL_EPARSE;
+    }
+    if (fail_at_nondigit && (endptr < (buf + buf_size)) && (*endptr != '\0')) {
+        /* non-digit encountered */
+        return CL_EPARSE;
+    }
+    /* success */
+    *result = num;
+    return CL_SUCCESS;
+}
+
 size_t cli_ldbtokenize(char *buffer, const char delim, const size_t token_count, const char **tokens, int token_skip)
 {
     size_t tokens_found, i;
