@@ -1,4 +1,5 @@
 /*
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2008 Sourcefire, Inc.
  *
  *  Authors: Alberto Wu
@@ -128,13 +129,13 @@ static int pefromupx (const char *src, uint32_t ssize, char *dst, uint32_t *dsiz
     return 0;
 
   while ((valign=magic[sectcnt++])) {
-    if ( ep - upx1 + valign <= ssize-5  &&    /* Wondering how we got so far?! */
+    if (CLI_ISCONTAINED(src, ssize - 5, src + ep - upx1 + valign - 2, 2) &&
 	 src[ep - upx1 + valign - 2] == '\x8d' && /* lea edi, ...                  */
 	 src[ep - upx1 + valign - 1] == '\xbe' )  /* ... [esi + offset]          */
       break;
   }
 
-  if (!valign && ep - upx1 + 0x80 < ssize-8) {
+  if (!valign && CLI_ISCONTAINED(src, ssize - 8, src + ep - upx1 + 0x80, 8)) {
     const char *pt = &src[ep - upx1 + 0x80];
     cli_dbgmsg("UPX: bad magic - scanning for imports\n");
     
@@ -186,7 +187,7 @@ static int pefromupx (const char *src, uint32_t ssize, char *dst, uint32_t *dsiz
 
   if (!pehdr) {
     uint32_t rebsz = PESALIGN(dend, 0x1000);
-    cli_dbgmsg("UPX: no luck - brutally crafing a reasonable PE\n");
+    cli_dbgmsg("UPX: no luck - brutally crafting a reasonable PE\n");
     if (!(newbuf = (char *)cli_calloc(rebsz+0x200, sizeof(char)))) {
       cli_dbgmsg("UPX: malloc failed - giving up rebuild\n");
       return 0;
@@ -542,14 +543,20 @@ int upx_inflate2e(const char *src, uint32_t ssize, char *dst, uint32_t *dsize, u
   return pefromupx (src, ssize, dst, dsize, ep, upx0, upx1, magic, dcur);
 }
 
-int upx_inflatelzma(const char *src, uint32_t ssize, char *dst, uint32_t *dsize, uint32_t upx0, uint32_t upx1, uint32_t ep) {
+int upx_inflatelzma(const char *src, uint32_t ssize, char *dst, uint32_t *dsize, uint32_t upx0, uint32_t upx1, uint32_t ep, uint32_t properties) {
   struct CLI_LZMA l;
   uint32_t magic[]={0xb16,0xb1e,0};
   unsigned char fake_lzmahdr[5];
 
   memset(&l, 0, sizeof(l));
   cli_writeint32(fake_lzmahdr + 1, *dsize);
-  *fake_lzmahdr = 3 /* lc */ + 9* ( 5* 2 /* pb */ + 0 /* lp */);
+  uint8_t lc = properties & 0xff;
+  uint8_t lp = (properties >> 8) & 0xff;
+  uint8_t pb = (properties >> 16) & 0xff; 
+  if (lc >= 9 || lp >= 5 || pb >= 5)
+      return -1;
+
+  *fake_lzmahdr = lc + 9* ( 5* pb + lp);
   l.next_in = fake_lzmahdr;
   l.avail_in = 5;
   if(cli_LzmaInit(&l, *dsize) != LZMA_RESULT_OK)
